On Mon, 6 Jul 2015, Sudeep Holla wrote:
> On 06/07/15 17:06, Thomas Gleixner wrote:
> 2. After boot I am seeing the below warning:
> 
> ------------[ cut here ]------------
> WARNING: CPU: 1 PID: 0 at kernel/time/hrtimer.c:1247 
> __hrtimer_run_queues+0x148/0x150()
> Modules linked in:
> CPU: 1 PID: 0 Comm: swapper/1 Not tainted 4.2.0-rc1 #573
> Hardware name: ARM Juno development board (r0) (DT)
> Call trace:
> [<ffffffc000089954>] dump_backtrace+0x0/0x124
> [<ffffffc000089a88>] show_stack+0x10/0x1c
> [<ffffffc0005d100c>] dump_stack+0x84/0xc8
> [<ffffffc0000b3f34>] warn_slowpath_common+0x98/0xd0
> [<ffffffc0000b402c>] warn_slowpath_null+0x14/0x20
> [<ffffffc000101eec>] __hrtimer_run_queues+0x144/0x150
> [<ffffffc0001028b4>] hrtimer_run_queues+0xb8/0xe8
> [<ffffffc000101714>] update_process_times+0x28/0x6c
> [<ffffffc00010d978>] tick_periodic+0x3c/0xb8
> [<ffffffc00010da1c>] tick_handle_periodic+0x28/0x94
> [<ffffffc0004d6154>] arch_timer_handler_phys+0x28/0x38
> [<ffffffc0000f5964>] handle_percpu_devid_irq+0x74/0x9c
> [<ffffffc0000f1668>] generic_handle_irq+0x30/0x4c
> [<ffffffc0000f1984>] __handle_domain_irq+0x5c/0xac
> [<ffffffc0000824a8>] gic_handle_irq+0x30/0x80

Bah. I fear Preeti's hrtimer-broadcast stuff recurses in the hrtimer
code.

tick_periodic()
  expire broadcast timer()
    handle_broadcast() {
      tick_periodic()
         expire_other_timer()
    }

  warning triggers       

Crap. Patch below should cure that.

Thanks,

        tglx
---
Index: tip/kernel/time/tick-broadcast.c
===================================================================
--- tip.orig/kernel/time/tick-broadcast.c
+++ tip/kernel/time/tick-broadcast.c
@@ -358,18 +358,21 @@ void tick_broadcast_control(enum tick_br
                tick_broadcast_forced = 1;
        case TICK_BROADCAST_ON:
                cpumask_set_cpu(cpu, tick_broadcast_on);
-               if (!cpumask_test_and_set_cpu(cpu, tick_broadcast_mask)) {
-                       /*
-                        * Only shutdown the cpu local device, if the
-                        * broadcast device exists and is in periodic
-                        * mode. The latter check prevents a hickup
-                        * during the switch from periodic to oneshot
-                        * mode.
-                        */
-                       if (bc != NULL && tick_broadcast_device.mode ==
-                           TICKDEV_MODE_PERIODIC)
-                               clockevents_shutdown(dev);
-               }
+
+               /*
+                * Only shutdown the cpu local device, if:
+                *
+                * - the broadcast device exists
+                * - the broadcast device is not a hrtimer based one
+                * - the broadcast device is in periodic mode to
+                *   avoid a hickup during switch to oneshot mode
+                */
+               if (!bc || (bc->features & CLOCK_EVT_FEAT_HRTIMER) ||
+                   tick_broadcast_device.mode != TICKDEV_MODE_PERIODIC)
+                       break;
+
+               if (!cpumask_test_and_set_cpu(cpu, tick_broadcast_mask))
+                       clockevents_shutdown(dev);
                break;
 
        case TICK_BROADCAST_OFF:
@@ -697,14 +700,18 @@ int __tick_broadcast_oneshot_control(enu
                 * shutdown.
                 */
                ret = broadcast_needs_cpu(bc, cpu);
+               if (ret)
+                       goto out;
 
                /*
-                * If the hrtimer broadcast check tells us that the
-                * cpu cannot go deep idle, or if the broadcast device
-                * is in periodic mode, we just return.
+                * If the broadcast device is in periodic mode, we
+                * return.
                 */
-               if (ret || tick_broadcast_device.mode == TICKDEV_MODE_PERIODIC)
-                       goto out;
+               if (tick_broadcast_device.mode == TICKDEV_MODE_PERIODIC) {
+                       /* If it is a hrtimer based broadcast, return busy */
+                       if (bc->features & CLOCK_EVT_FEAT_HRTIMER)
+                               ret = -EBUSY;
+               }
 
                if (!cpumask_test_and_set_cpu(cpu, 
tick_broadcast_oneshot_mask)) {
                        WARN_ON_ONCE(cpumask_test_cpu(cpu, 
tick_broadcast_pending_mask));


--
To unsubscribe from this list: send the line "unsubscribe linux-kernel" in
the body of a message to majord...@vger.kernel.org
More majordomo info at  http://vger.kernel.org/majordomo-info.html
Please read the FAQ at  http://www.tux.org/lkml/

Reply via email to